package com.example.nto.controller;

import com.example.nto.dto.entity.OfficeDTO;
import com.example.nto.service.OfficeService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("api/v1/offices")
public class OfficeController {
    private final OfficeService officeService;

    @GetMapping
    public ResponseEntity<List<OfficeDTO>> getAll() {
        return ResponseEntity.ok(officeService.getAllOffice());
    }

    @GetMapping("/sorted/distance")
    public ResponseEntity<List<OfficeDTO>> getAllSortedByDistance(
            @RequestParam(name = "latitude") double latitude,
            @RequestParam(name = "longitude") double longitude
    ) {
        return ResponseEntity.ok(officeService.getAllSortedDistance(latitude, longitude));
    }

    @GetMapping("/{officeId}")
    public ResponseEntity<OfficeDTO> getById(@PathVariable long officeId) {
        return ResponseEntity.ok(officeService.getById(officeId));
    }

    @PostMapping
    public ResponseEntity<OfficeDTO> createOffice(@RequestBody OfficeDTO officeDTO) {
        return ResponseEntity.ok(officeService.create(officeDTO));
    }

    @PutMapping("/{officeId}")
    public ResponseEntity<OfficeDTO> updateOffice(@PathVariable long officeId, @RequestBody OfficeDTO officeDTO) {
        return ResponseEntity.ok(officeService.update(officeId, officeDTO));
    }

    @DeleteMapping("/{officeId}")
    public ResponseEntity<Void> deleteOffice(@PathVariable long officeId) {
        officeService.delete(officeId);
        return ResponseEntity.noContent().build();
    }
}
